/********************************************************************************
** Form generated from reading UI file 'extData.ui'
**
** Created by: Qt User Interface Compiler version 4.8.6
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXTDATA_H
#define UI_EXTDATA_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QDialog>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QPlainTextEdit>
#include <QtGui/QPushButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DataDialog
{
public:
    QPlainTextEdit *txtDataInput;
    QWidget *horizontalLayoutWidget;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnFromFile;
    QPushButton *btnReset;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnOK;
    QPushButton *btnCancel;
    QLabel *label;

    void setupUi(QDialog *DataDialog)
    {
        if (DataDialog->objectName().isEmpty())
            DataDialog->setObjectName(QString::fromUtf8("DataDialog"));
        DataDialog->resize(400, 300);
        DataDialog->setModal(true);
        txtDataInput = new QPlainTextEdit(DataDialog);
        txtDataInput->setObjectName(QString::fromUtf8("txtDataInput"));
        txtDataInput->setGeometry(QRect(10, 20, 381, 221));
        horizontalLayoutWidget = new QWidget(DataDialog);
        horizontalLayoutWidget->setObjectName(QString::fromUtf8("horizontalLayoutWidget"));
        horizontalLayoutWidget->setGeometry(QRect(10, 250, 381, 41));
        horizontalLayout = new QHBoxLayout(horizontalLayoutWidget);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        btnFromFile = new QPushButton(horizontalLayoutWidget);
        btnFromFile->setObjectName(QString::fromUtf8("btnFromFile"));

        horizontalLayout->addWidget(btnFromFile);

        btnReset = new QPushButton(horizontalLayoutWidget);
        btnReset->setObjectName(QString::fromUtf8("btnReset"));

        horizontalLayout->addWidget(btnReset);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btnOK = new QPushButton(horizontalLayoutWidget);
        btnOK->setObjectName(QString::fromUtf8("btnOK"));

        horizontalLayout->addWidget(btnOK);

        btnCancel = new QPushButton(horizontalLayoutWidget);
        btnCancel->setObjectName(QString::fromUtf8("btnCancel"));

        horizontalLayout->addWidget(btnCancel);

        label = new QLabel(DataDialog);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(10, 0, 56, 16));

        retranslateUi(DataDialog);

        QMetaObject::connectSlotsByName(DataDialog);
    } // setupUi

    void retranslateUi(QDialog *DataDialog)
    {
        DataDialog->setWindowTitle(QApplication::translate("DataDialog", "Input Data", 0, QApplication::UnicodeUTF8));
        btnFromFile->setText(QApplication::translate("DataDialog", "From File...", 0, QApplication::UnicodeUTF8));
        btnReset->setText(QApplication::translate("DataDialog", "Reset", 0, QApplication::UnicodeUTF8));
        btnOK->setText(QApplication::translate("DataDialog", "OK", 0, QApplication::UnicodeUTF8));
        btnCancel->setText(QApplication::translate("DataDialog", "Cancel", 0, QApplication::UnicodeUTF8));
        label->setText(QApplication::translate("DataDialog", "Data", 0, QApplication::UnicodeUTF8));
    } // retranslateUi

};

namespace Ui {
    class DataDialog: public Ui_DataDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXTDATA_H
