/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/zint-2.4.3/frontend_qt4/mainwindow.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_MainWindow[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       1,   84, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: signature, parameters, type, tag, flags
      12,   11,   11,   11, 0x0a,
      29,   11,   11,   11, 0x0a,
      46,   11,   11,   11, 0x0a,
      67,   11,   11,   11, 0x0a,
      88,   11,   11,   11, 0x0a,
     107,   11,   11,   11, 0x0a,
     129,   11,   11,   11, 0x0a,
     144,   11,   11,   11, 0x0a,
     170,   11,  165,   11, 0x08,
     177,   11,   11,   11, 0x08,
     185,   11,   11,   11, 0x08,
     196,   11,   11,   11, 0x08,
     213,   11,  209,   11, 0x08,
     232,   11,  209,   11, 0x08,

 // enums: name, flags, count, data
     255, 0x0,   61,   88,

 // enum data: key, value
     268, uint(MainWindow::AUSREDIRECT),
     280, uint(MainWindow::AUSREPLY),
     289, uint(MainWindow::AUSROUTE),
     298, uint(MainWindow::AUSPOST),
     306, uint(MainWindow::AZTEC),
     312, uint(MainWindow::AZRUNE),
     319, uint(MainWindow::CHANNEL),
     327, uint(MainWindow::CODABAR),
     335, uint(MainWindow::CODE11),
     342, uint(MainWindow::CODE128),
     350, uint(MainWindow::CODE16K),
     358, uint(MainWindow::C25LOGIC),
     367, uint(MainWindow::C25IATA),
     375, uint(MainWindow::C25IND),
     382, uint(MainWindow::C25INTER),
     391, uint(MainWindow::C25MATRIX),
     401, uint(MainWindow::CODE32),
     408, uint(MainWindow::CODE39),
     415, uint(MainWindow::EXCODE39),
     424, uint(MainWindow::CODE49),
     431, uint(MainWindow::CODE93),
     438, uint(MainWindow::CODE_ONE),
     447, uint(MainWindow::RSS14),
     453, uint(MainWindow::RSS_EXP),
     461, uint(MainWindow::RSS_EXPSTACK),
     474, uint(MainWindow::RSS_LTD),
     482, uint(MainWindow::RSS14STACK),
     493, uint(MainWindow::RSS14STACK_OMNI),
     509, uint(MainWindow::DATAMATRIX),
     520, uint(MainWindow::DPIDENT),
     528, uint(MainWindow::DPLEIT),
     535, uint(MainWindow::KIX),
     539, uint(MainWindow::EAN14),
     545, uint(MainWindow::EANX),
     550, uint(MainWindow::FIM),
     554, uint(MainWindow::FLAT),
     559, uint(MainWindow::GRIDMATRIX),
     570, uint(MainWindow::ITF14),
     576, uint(MainWindow::ISBNX),
     582, uint(MainWindow::JAPANPOST),
     592, uint(MainWindow::KOREAPOST),
     602, uint(MainWindow::LOGMARS),
     610, uint(MainWindow::MAXICODE),
     619, uint(MainWindow::MICROPDF417),
     631, uint(MainWindow::MICROQR),
     639, uint(MainWindow::MSI_PLESSEY),
     651, uint(MainWindow::NVE18),
     657, uint(MainWindow::PDF417),
     664, uint(MainWindow::PHARMA),
     671, uint(MainWindow::PHARMA_TWO),
     682, uint(MainWindow::PZN),
     686, uint(MainWindow::PLANET),
     693, uint(MainWindow::POSTNET),
     701, uint(MainWindow::QRCODE),
     708, uint(MainWindow::RM4SCC),
     715, uint(MainWindow::TELEPEN),
     723, uint(MainWindow::TELEPEN_NUM),
     735, uint(MainWindow::PLESSEY),
     743, uint(MainWindow::UPCA),
     748, uint(MainWindow::UPCE),
     753, uint(MainWindow::ONECODE),

       0        // eod
};

static const char qt_meta_stringdata_MainWindow[] = {
    "MainWindow\0\0update_preview()\0"
    "change_options()\0on_fgcolor_clicked()\0"
    "on_bgcolor_clicked()\0composite_enable()\0"
    "composite_ean_check()\0maxi_primary()\0"
    "change_print_scale()\0bool\0save()\0"
    "about()\0quit_now()\0reset_view()\0int\0"
    "open_data_dialog()\0open_sequence_dialog()\0"
    "BarcodeTypes\0AUSREDIRECT\0AUSREPLY\0"
    "AUSROUTE\0AUSPOST\0AZTEC\0AZRUNE\0CHANNEL\0"
    "CODABAR\0CODE11\0CODE128\0CODE16K\0C25LOGIC\0"
    "C25IATA\0C25IND\0C25INTER\0C25MATRIX\0"
    "CODE32\0CODE39\0EXCODE39\0CODE49\0CODE93\0"
    "CODE_ONE\0RSS14\0RSS_EXP\0RSS_EXPSTACK\0"
    "RSS_LTD\0RSS14STACK\0RSS14STACK_OMNI\0"
    "DATAMATRIX\0DPIDENT\0DPLEIT\0KIX\0EAN14\0"
    "EANX\0FIM\0FLAT\0GRIDMATRIX\0ITF14\0ISBNX\0"
    "JAPANPOST\0KOREAPOST\0LOGMARS\0MAXICODE\0"
    "MICROPDF417\0MICROQR\0MSI_PLESSEY\0NVE18\0"
    "PDF417\0PHARMA\0PHARMA_TWO\0PZN\0PLANET\0"
    "POSTNET\0QRCODE\0RM4SCC\0TELEPEN\0TELEPEN_NUM\0"
    "PLESSEY\0UPCA\0UPCE\0ONECODE\0"
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        MainWindow *_t = static_cast<MainWindow *>(_o);
        switch (_id) {
        case 0: _t->update_preview(); break;
        case 1: _t->change_options(); break;
        case 2: _t->on_fgcolor_clicked(); break;
        case 3: _t->on_bgcolor_clicked(); break;
        case 4: _t->composite_enable(); break;
        case 5: _t->composite_ean_check(); break;
        case 6: _t->maxi_primary(); break;
        case 7: _t->change_print_scale(); break;
        case 8: { bool _r = _t->save();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 9: _t->about(); break;
        case 10: _t->quit_now(); break;
        case 11: _t->reset_view(); break;
        case 12: { int _r = _t->open_data_dialog();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = _r; }  break;
        case 13: { int _r = _t->open_sequence_dialog();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = _r; }  break;
        default: ;
        }
    }
}

const QMetaObjectExtraData MainWindow::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject MainWindow::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_MainWindow,
      qt_meta_data_MainWindow, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &MainWindow::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow))
        return static_cast<void*>(const_cast< MainWindow*>(this));
    return QWidget::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
