/********************************************************************************
** Form generated from reading UI file 'extExport.ui'
**
** Created by: Qt User Interface Compiler version 4.8.6
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXTEXPORT_H
#define UI_EXTEXPORT_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QComboBox>
#include <QtGui/QDialog>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>
#include <QtGui/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_ExportDialog
{
public:
    QLineEdit *linDestPath;
    QLineEdit *linPrefix;
    QComboBox *cmbFileName;
    QComboBox *cmbFileFormat;
    QToolButton *btnDestPath;
    QLabel *label;
    QLabel *label_2;
    QLabel *label_3;
    QLabel *label_4;
    QPushButton *btnCancel;
    QPushButton *btnOK;

    void setupUi(QDialog *ExportDialog)
    {
        if (ExportDialog->objectName().isEmpty())
            ExportDialog->setObjectName(QString::fromUtf8("ExportDialog"));
        ExportDialog->resize(450, 170);
        linDestPath = new QLineEdit(ExportDialog);
        linDestPath->setObjectName(QString::fromUtf8("linDestPath"));
        linDestPath->setGeometry(QRect(140, 10, 261, 22));
        linPrefix = new QLineEdit(ExportDialog);
        linPrefix->setObjectName(QString::fromUtf8("linPrefix"));
        linPrefix->setGeometry(QRect(140, 40, 301, 22));
        cmbFileName = new QComboBox(ExportDialog);
        cmbFileName->setObjectName(QString::fromUtf8("cmbFileName"));
        cmbFileName->setGeometry(QRect(140, 70, 301, 22));
        cmbFileFormat = new QComboBox(ExportDialog);
        cmbFileFormat->setObjectName(QString::fromUtf8("cmbFileFormat"));
        cmbFileFormat->setGeometry(QRect(140, 100, 301, 22));
        btnDestPath = new QToolButton(ExportDialog);
        btnDestPath->setObjectName(QString::fromUtf8("btnDestPath"));
        btnDestPath->setGeometry(QRect(410, 10, 30, 25));
        label = new QLabel(ExportDialog);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(10, 10, 121, 16));
        label_2 = new QLabel(ExportDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setGeometry(QRect(10, 40, 111, 16));
        label_3 = new QLabel(ExportDialog);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setGeometry(QRect(10, 70, 111, 16));
        label_4 = new QLabel(ExportDialog);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setGeometry(QRect(10, 100, 111, 16));
        btnCancel = new QPushButton(ExportDialog);
        btnCancel->setObjectName(QString::fromUtf8("btnCancel"));
        btnCancel->setGeometry(QRect(360, 130, 80, 26));
        btnOK = new QPushButton(ExportDialog);
        btnOK->setObjectName(QString::fromUtf8("btnOK"));
        btnOK->setGeometry(QRect(270, 130, 80, 26));

        retranslateUi(ExportDialog);

        QMetaObject::connectSlotsByName(ExportDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportDialog)
    {
        ExportDialog->setWindowTitle(QApplication::translate("ExportDialog", "Export Barcodes", 0, QApplication::UnicodeUTF8));
        linPrefix->setText(QApplication::translate("ExportDialog", "bcs_", 0, QApplication::UnicodeUTF8));
        cmbFileName->clear();
        cmbFileName->insertItems(0, QStringList()
         << QApplication::translate("ExportDialog", "Same as Data", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("ExportDialog", "Serial Number", 0, QApplication::UnicodeUTF8)
        );
        cmbFileFormat->clear();
        cmbFileFormat->insertItems(0, QStringList()
         << QApplication::translate("ExportDialog", "Portable Network Graphic (*.png)", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("ExportDialog", "Encapsulated Post Script (*.eps)", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("ExportDialog", "Scalable Vector Graphic (*.svg)", 0, QApplication::UnicodeUTF8)
        );
        btnDestPath->setText(QApplication::translate("ExportDialog", "...", 0, QApplication::UnicodeUTF8));
        label->setText(QApplication::translate("ExportDialog", "Destination Path:", 0, QApplication::UnicodeUTF8));
        label_2->setText(QApplication::translate("ExportDialog", "File Name Prefix:", 0, QApplication::UnicodeUTF8));
        label_3->setText(QApplication::translate("ExportDialog", "File Name:", 0, QApplication::UnicodeUTF8));
        label_4->setText(QApplication::translate("ExportDialog", "File Format:", 0, QApplication::UnicodeUTF8));
        btnCancel->setText(QApplication::translate("ExportDialog", "Cancel", 0, QApplication::UnicodeUTF8));
        btnOK->setText(QApplication::translate("ExportDialog", "OK", 0, QApplication::UnicodeUTF8));
    } // retranslateUi

};

namespace Ui {
    class ExportDialog: public Ui_ExportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXTEXPORT_H
