/********************************************************************************
** Form generated from reading UI file 'extSequence.ui'
**
** Created by: Qt User Interface Compiler version 4.8.6
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXTSEQUENCE_H
#define UI_EXTSEQUENCE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QDialog>
#include <QtGui/QGroupBox>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QLineEdit>
#include <QtGui/QPlainTextEdit>
#include <QtGui/QPushButton>

QT_BEGIN_NAMESPACE

class Ui_SequenceDialog
{
public:
    QGroupBox *groupBox;
    QPushButton *btnExport;
    QPushButton *btnCreate;
    QLineEdit *linStartVal;
    QLineEdit *linFormat;
    QLineEdit *linEndVal;
    QPushButton *btnImport;
    QLabel *label;
    QLabel *label_2;
    QLabel *lblIncrement;
    QLabel *label_4;
    QLabel *label_5;
    QLabel *label_6;
    QLabel *label_7;
    QLineEdit *linIncVal;
    QGroupBox *groupBox_2;
    QPlainTextEdit *txtPreview;
    QPushButton *btnReset;
    QPushButton *btnClose;

    void setupUi(QDialog *SequenceDialog)
    {
        if (SequenceDialog->objectName().isEmpty())
            SequenceDialog->setObjectName(QString::fromUtf8("SequenceDialog"));
        SequenceDialog->resize(530, 300);
        SequenceDialog->setModal(true);
        groupBox = new QGroupBox(SequenceDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        groupBox->setGeometry(QRect(10, 10, 261, 241));
        btnExport = new QPushButton(groupBox);
        btnExport->setObjectName(QString::fromUtf8("btnExport"));
        btnExport->setEnabled(false);
        btnExport->setGeometry(QRect(140, 200, 111, 26));
        btnCreate = new QPushButton(groupBox);
        btnCreate->setObjectName(QString::fromUtf8("btnCreate"));
        btnCreate->setGeometry(QRect(140, 140, 111, 26));
        linStartVal = new QLineEdit(groupBox);
        linStartVal->setObjectName(QString::fromUtf8("linStartVal"));
        linStartVal->setGeometry(QRect(140, 20, 111, 22));
        linStartVal->setFrame(true);
        linFormat = new QLineEdit(groupBox);
        linFormat->setObjectName(QString::fromUtf8("linFormat"));
        linFormat->setGeometry(QRect(140, 110, 111, 22));
        linEndVal = new QLineEdit(groupBox);
        linEndVal->setObjectName(QString::fromUtf8("linEndVal"));
        linEndVal->setGeometry(QRect(140, 50, 111, 22));
        btnImport = new QPushButton(groupBox);
        btnImport->setObjectName(QString::fromUtf8("btnImport"));
        btnImport->setGeometry(QRect(140, 170, 111, 26));
        label = new QLabel(groupBox);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(10, 20, 151, 20));
        label_2 = new QLabel(groupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setGeometry(QRect(10, 50, 171, 16));
        lblIncrement = new QLabel(groupBox);
        lblIncrement->setObjectName(QString::fromUtf8("lblIncrement"));
        lblIncrement->setGeometry(QRect(10, 80, 171, 16));
        label_4 = new QLabel(groupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setGeometry(QRect(10, 110, 171, 16));
        label_5 = new QLabel(groupBox);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setGeometry(QRect(10, 140, 161, 16));
        label_6 = new QLabel(groupBox);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        label_6->setGeometry(QRect(10, 170, 161, 16));
        label_7 = new QLabel(groupBox);
        label_7->setObjectName(QString::fromUtf8("label_7"));
        label_7->setGeometry(QRect(10, 200, 161, 16));
        linIncVal = new QLineEdit(groupBox);
        linIncVal->setObjectName(QString::fromUtf8("linIncVal"));
        linIncVal->setGeometry(QRect(140, 80, 113, 22));
        groupBox_2 = new QGroupBox(SequenceDialog);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        groupBox_2->setGeometry(QRect(280, 10, 241, 241));
        txtPreview = new QPlainTextEdit(groupBox_2);
        txtPreview->setObjectName(QString::fromUtf8("txtPreview"));
        txtPreview->setGeometry(QRect(10, 20, 221, 171));
        btnReset = new QPushButton(groupBox_2);
        btnReset->setObjectName(QString::fromUtf8("btnReset"));
        btnReset->setGeometry(QRect(150, 200, 80, 26));
        btnClose = new QPushButton(SequenceDialog);
        btnClose->setObjectName(QString::fromUtf8("btnClose"));
        btnClose->setGeometry(QRect(440, 260, 80, 26));

        retranslateUi(SequenceDialog);

        QMetaObject::connectSlotsByName(SequenceDialog);
    } // setupUi

    void retranslateUi(QDialog *SequenceDialog)
    {
        SequenceDialog->setWindowTitle(QApplication::translate("SequenceDialog", "Sequence Export", 0, QApplication::UnicodeUTF8));
        groupBox->setTitle(QApplication::translate("SequenceDialog", "Create Sequence", 0, QApplication::UnicodeUTF8));
        btnExport->setText(QApplication::translate("SequenceDialog", "Export...", 0, QApplication::UnicodeUTF8));
        btnCreate->setText(QApplication::translate("SequenceDialog", "Create", 0, QApplication::UnicodeUTF8));
        linStartVal->setText(QApplication::translate("SequenceDialog", "1", 0, QApplication::UnicodeUTF8));
        linFormat->setText(QApplication::translate("SequenceDialog", "$$$$$$", 0, QApplication::UnicodeUTF8));
        linEndVal->setText(QApplication::translate("SequenceDialog", "10", 0, QApplication::UnicodeUTF8));
        btnImport->setText(QApplication::translate("SequenceDialog", "Import...", 0, QApplication::UnicodeUTF8));
        label->setText(QApplication::translate("SequenceDialog", "Start Value:", 0, QApplication::UnicodeUTF8));
        label_2->setText(QApplication::translate("SequenceDialog", "End Value:", 0, QApplication::UnicodeUTF8));
        lblIncrement->setText(QApplication::translate("SequenceDialog", "Increment By:", 0, QApplication::UnicodeUTF8));
        label_4->setText(QApplication::translate("SequenceDialog", "Format:", 0, QApplication::UnicodeUTF8));
        label_5->setText(QApplication::translate("SequenceDialog", "Sequence:", 0, QApplication::UnicodeUTF8));
        label_6->setText(QApplication::translate("SequenceDialog", "Sequence File:", 0, QApplication::UnicodeUTF8));
        label_7->setText(QApplication::translate("SequenceDialog", "Generate Bar Codes:", 0, QApplication::UnicodeUTF8));
        linIncVal->setText(QApplication::translate("SequenceDialog", "1", 0, QApplication::UnicodeUTF8));
        groupBox_2->setTitle(QApplication::translate("SequenceDialog", "Sequence Preview", 0, QApplication::UnicodeUTF8));
        btnReset->setText(QApplication::translate("SequenceDialog", "Reset", 0, QApplication::UnicodeUTF8));
        btnClose->setText(QApplication::translate("SequenceDialog", "Close", 0, QApplication::UnicodeUTF8));
    } // retranslateUi

};

namespace Ui {
    class SequenceDialog: public Ui_SequenceDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXTSEQUENCE_H
