/********************************************************************************
** Form generated from reading UI file 'mainWindow.ui'
**
** Created by: Qt User Interface Compiler version 4.8.6
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QComboBox>
#include <QtGui/QDoubleSpinBox>
#include <QtGui/QGraphicsView>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QSpinBox>
#include <QtGui/QTabWidget>
#include <QtGui/QTextEdit>
#include <QtGui/QToolButton>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>

QT_BEGIN_NAMESPACE

class Ui_mainWindow
{
public:
    QVBoxLayout *verticalLayout_5;
    QGraphicsView *view;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_5;
    QLabel *lblSymbology;
    QComboBox *bstyle;
    QTabWidget *tabMain;
    QWidget *tabGeneral;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *grpData;
    QHBoxLayout *horizontalLayout_4;
    QLineEdit *txtData;
    QToolButton *btnMoreData;
    QPushButton *btnSequence;
    QGroupBox *grpComposite;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *chkComposite;
    QSpacerItem *horizontalSpacer;
    QLabel *lblCompType;
    QComboBox *cmbCompType;
    QVBoxLayout *verticalLayout;
    QLabel *lblComposite;
    QTextEdit *txtComposite;
    QWidget *tabAppear;
    QVBoxLayout *verticalLayout_4;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QLabel *label_8;
    QLabel *lblWhitespace;
    QLabel *lblScale;
    QSpinBox *heightb;
    QSpinBox *bwidth;
    QSpinBox *spnWhitespace;
    QDoubleSpinBox *spnScale;
    QLabel *label_9;
    QComboBox *btype;
    QPushButton *fgcolor;
    QPushButton *bgcolor;
    QPushButton *btnReset;
    QCheckBox *chkHRTHide;
    QSpacerItem *verticalSpacer;
    QGridLayout *gridLayout_2;
    QPushButton *btnAbout;
    QPushButton *btnSave;
    QPushButton *btnExit;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *mainWindow)
    {
        if (mainWindow->objectName().isEmpty())
            mainWindow->setObjectName(QString::fromUtf8("mainWindow"));
        mainWindow->setWindowModality(Qt::NonModal);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mainWindow->sizePolicy().hasHeightForWidth());
        mainWindow->setSizePolicy(sizePolicy);
        mainWindow->setMinimumSize(QSize(590, 600));
        mainWindow->setMaximumSize(QSize(590, 600));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/zint.png"), QSize(), QIcon::Normal, QIcon::Off);
        mainWindow->setWindowIcon(icon);
        mainWindow->setAutoFillBackground(false);
        mainWindow->setStyleSheet(QString::fromUtf8(""));
        mainWindow->setLocale(QLocale(QLocale::English, QLocale::UnitedStates));
        verticalLayout_5 = new QVBoxLayout(mainWindow);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        view = new QGraphicsView(mainWindow);
        view->setObjectName(QString::fromUtf8("view"));
        sizePolicy.setHeightForWidth(view->sizePolicy().hasHeightForWidth());
        view->setSizePolicy(sizePolicy);
        view->setMinimumSize(QSize(0, 0));
        view->setMaximumSize(QSize(1000, 1000));
        view->setAutoFillBackground(false);

        verticalLayout_5->addWidget(view);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        lblSymbology = new QLabel(mainWindow);
        lblSymbology->setObjectName(QString::fromUtf8("lblSymbology"));
        lblSymbology->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout_5->addWidget(lblSymbology);

        bstyle = new QComboBox(mainWindow);
        bstyle->setObjectName(QString::fromUtf8("bstyle"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(bstyle->sizePolicy().hasHeightForWidth());
        bstyle->setSizePolicy(sizePolicy1);

        horizontalLayout_5->addWidget(bstyle);


        verticalLayout_6->addLayout(horizontalLayout_5);

        tabMain = new QTabWidget(mainWindow);
        tabMain->setObjectName(QString::fromUtf8("tabMain"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(tabMain->sizePolicy().hasHeightForWidth());
        tabMain->setSizePolicy(sizePolicy2);
        tabMain->setMaximumSize(QSize(16777215, 350));
        tabMain->setAutoFillBackground(false);
        tabMain->setStyleSheet(QString::fromUtf8(""));
        tabGeneral = new QWidget();
        tabGeneral->setObjectName(QString::fromUtf8("tabGeneral"));
        verticalLayout_3 = new QVBoxLayout(tabGeneral);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        grpData = new QGroupBox(tabGeneral);
        grpData->setObjectName(QString::fromUtf8("grpData"));
        sizePolicy2.setHeightForWidth(grpData->sizePolicy().hasHeightForWidth());
        grpData->setSizePolicy(sizePolicy2);
        horizontalLayout_4 = new QHBoxLayout(grpData);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        txtData = new QLineEdit(grpData);
        txtData->setObjectName(QString::fromUtf8("txtData"));

        horizontalLayout_4->addWidget(txtData);

        btnMoreData = new QToolButton(grpData);
        btnMoreData->setObjectName(QString::fromUtf8("btnMoreData"));
        btnMoreData->setMinimumSize(QSize(0, 24));

        horizontalLayout_4->addWidget(btnMoreData);

        btnSequence = new QPushButton(grpData);
        btnSequence->setObjectName(QString::fromUtf8("btnSequence"));
        btnSequence->setMinimumSize(QSize(0, 24));

        horizontalLayout_4->addWidget(btnSequence);


        verticalLayout_3->addWidget(grpData);

        grpComposite = new QGroupBox(tabGeneral);
        grpComposite->setObjectName(QString::fromUtf8("grpComposite"));
        grpComposite->setEnabled(true);
        sizePolicy2.setHeightForWidth(grpComposite->sizePolicy().hasHeightForWidth());
        grpComposite->setSizePolicy(sizePolicy2);
        grpComposite->setMinimumSize(QSize(0, 0));
        grpComposite->setMaximumSize(QSize(16777215, 400));
        verticalLayout_2 = new QVBoxLayout(grpComposite);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        chkComposite = new QCheckBox(grpComposite);
        chkComposite->setObjectName(QString::fromUtf8("chkComposite"));

        horizontalLayout_2->addWidget(chkComposite);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        lblCompType = new QLabel(grpComposite);
        lblCompType->setObjectName(QString::fromUtf8("lblCompType"));
        lblCompType->setEnabled(false);

        horizontalLayout_2->addWidget(lblCompType);

        cmbCompType = new QComboBox(grpComposite);
        cmbCompType->setObjectName(QString::fromUtf8("cmbCompType"));
        cmbCompType->setEnabled(false);

        horizontalLayout_2->addWidget(cmbCompType);


        verticalLayout_2->addLayout(horizontalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        lblComposite = new QLabel(grpComposite);
        lblComposite->setObjectName(QString::fromUtf8("lblComposite"));
        lblComposite->setEnabled(false);

        verticalLayout->addWidget(lblComposite);

        txtComposite = new QTextEdit(grpComposite);
        txtComposite->setObjectName(QString::fromUtf8("txtComposite"));
        txtComposite->setEnabled(false);
        QSizePolicy sizePolicy3(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(txtComposite->sizePolicy().hasHeightForWidth());
        txtComposite->setSizePolicy(sizePolicy3);
        txtComposite->setMaximumSize(QSize(16777215, 60));
        txtComposite->setAcceptRichText(false);

        verticalLayout->addWidget(txtComposite);


        verticalLayout_2->addLayout(verticalLayout);


        verticalLayout_3->addWidget(grpComposite);

        tabMain->addTab(tabGeneral, QString());
        tabAppear = new QWidget();
        tabAppear->setObjectName(QString::fromUtf8("tabAppear"));
        verticalLayout_4 = new QVBoxLayout(tabAppear);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_3 = new QLabel(tabAppear);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        label_8 = new QLabel(tabAppear);
        label_8->setObjectName(QString::fromUtf8("label_8"));
        label_8->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(label_8, 1, 0, 1, 1);

        lblWhitespace = new QLabel(tabAppear);
        lblWhitespace->setObjectName(QString::fromUtf8("lblWhitespace"));
        lblWhitespace->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(lblWhitespace, 2, 0, 1, 1);

        lblScale = new QLabel(tabAppear);
        lblScale->setObjectName(QString::fromUtf8("lblScale"));
        lblScale->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(lblScale, 3, 0, 1, 1);

        heightb = new QSpinBox(tabAppear);
        heightb->setObjectName(QString::fromUtf8("heightb"));
        heightb->setMaximum(300);
        heightb->setValue(50);

        gridLayout->addWidget(heightb, 0, 1, 1, 1);

        bwidth = new QSpinBox(tabAppear);
        bwidth->setObjectName(QString::fromUtf8("bwidth"));
        bwidth->setMinimum(0);
        bwidth->setMaximum(16);
        bwidth->setValue(0);

        gridLayout->addWidget(bwidth, 1, 1, 1, 1);

        spnWhitespace = new QSpinBox(tabAppear);
        spnWhitespace->setObjectName(QString::fromUtf8("spnWhitespace"));

        gridLayout->addWidget(spnWhitespace, 2, 1, 1, 1);

        spnScale = new QDoubleSpinBox(tabAppear);
        spnScale->setObjectName(QString::fromUtf8("spnScale"));
        spnScale->setDecimals(2);
        spnScale->setMinimum(0.5);
        spnScale->setMaximum(99.5);
        spnScale->setSingleStep(0.5);
        spnScale->setValue(1);

        gridLayout->addWidget(spnScale, 3, 1, 1, 1);

        label_9 = new QLabel(tabAppear);
        label_9->setObjectName(QString::fromUtf8("label_9"));
        label_9->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(label_9, 0, 2, 1, 1);

        btype = new QComboBox(tabAppear);
        btype->setObjectName(QString::fromUtf8("btype"));

        gridLayout->addWidget(btype, 0, 3, 1, 1);

        fgcolor = new QPushButton(tabAppear);
        fgcolor->setObjectName(QString::fromUtf8("fgcolor"));

        gridLayout->addWidget(fgcolor, 1, 3, 1, 1);

        bgcolor = new QPushButton(tabAppear);
        bgcolor->setObjectName(QString::fromUtf8("bgcolor"));

        gridLayout->addWidget(bgcolor, 2, 3, 1, 1);

        btnReset = new QPushButton(tabAppear);
        btnReset->setObjectName(QString::fromUtf8("btnReset"));

        gridLayout->addWidget(btnReset, 3, 3, 1, 1);


        verticalLayout_4->addLayout(gridLayout);

        chkHRTHide = new QCheckBox(tabAppear);
        chkHRTHide->setObjectName(QString::fromUtf8("chkHRTHide"));
        chkHRTHide->setChecked(true);

        verticalLayout_4->addWidget(chkHRTHide);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Minimum);

        verticalLayout_4->addItem(verticalSpacer);

        tabMain->addTab(tabAppear, QString());

        verticalLayout_6->addWidget(tabMain);


        horizontalLayout->addLayout(verticalLayout_6);


        verticalLayout_5->addLayout(horizontalLayout);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        btnAbout = new QPushButton(mainWindow);
        btnAbout->setObjectName(QString::fromUtf8("btnAbout"));
        btnAbout->setMinimumSize(QSize(100, 0));

        gridLayout_2->addWidget(btnAbout, 0, 0, 1, 1);

        btnSave = new QPushButton(mainWindow);
        btnSave->setObjectName(QString::fromUtf8("btnSave"));
        btnSave->setMinimumSize(QSize(100, 0));

        gridLayout_2->addWidget(btnSave, 0, 4, 1, 1);

        btnExit = new QPushButton(mainWindow);
        btnExit->setObjectName(QString::fromUtf8("btnExit"));
        btnExit->setMinimumSize(QSize(100, 0));

        gridLayout_2->addWidget(btnExit, 0, 5, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 0, 1, 1, 1);


        verticalLayout_5->addLayout(gridLayout_2);

#ifndef QT_NO_SHORTCUT
        lblSymbology->setBuddy(bstyle);
        lblCompType->setBuddy(cmbCompType);
        lblComposite->setBuddy(txtComposite);
        label_3->setBuddy(heightb);
        label_8->setBuddy(bwidth);
        lblWhitespace->setBuddy(spnWhitespace);
        lblScale->setBuddy(spnScale);
        label_9->setBuddy(btype);
#endif // QT_NO_SHORTCUT
        QWidget::setTabOrder(tabMain, txtData);
        QWidget::setTabOrder(txtData, chkComposite);
        QWidget::setTabOrder(chkComposite, cmbCompType);
        QWidget::setTabOrder(cmbCompType, txtComposite);

        retranslateUi(mainWindow);

        tabMain->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(mainWindow);
    } // setupUi

    void retranslateUi(QWidget *mainWindow)
    {
        mainWindow->setWindowTitle(QApplication::translate("mainWindow", "Zint Barcode Studio 2.4", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        view->setToolTip(QApplication::translate("mainWindow", "Resulting barcode shown here", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        lblSymbology->setText(QApplication::translate("mainWindow", "S&ymbology:", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        bstyle->setToolTip(QApplication::translate("mainWindow", "Type of barcode", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        grpData->setTitle(QApplication::translate("mainWindow", "&Data to Encode", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        txtData->setToolTip(QApplication::translate("mainWindow", "Data to encode", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        txtData->setText(QApplication::translate("mainWindow", "Your Data Here!", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        btnMoreData->setToolTip(QApplication::translate("mainWindow", "Encode more data\n"
"or import from file", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        btnMoreData->setText(QApplication::translate("mainWindow", "...", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        btnSequence->setToolTip(QApplication::translate("mainWindow", "Encode a sequence", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        btnSequence->setText(QApplication::translate("mainWindow", "123..", 0, QApplication::UnicodeUTF8));
        grpComposite->setTitle(QApplication::translate("mainWindow", "Composite Code", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        chkComposite->setToolTip(QApplication::translate("mainWindow", "Create a composite symbol", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        chkComposite->setText(QApplication::translate("mainWindow", "Add &2D Component", 0, QApplication::UnicodeUTF8));
        lblCompType->setText(QApplication::translate("mainWindow", "Component &Type:", 0, QApplication::UnicodeUTF8));
        cmbCompType->clear();
        cmbCompType->insertItems(0, QStringList()
         << QApplication::translate("mainWindow", "Automatic", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("mainWindow", "CC-A", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("mainWindow", "CC-B", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("mainWindow", "CC-C", 0, QApplication::UnicodeUTF8)
        );
#ifndef QT_NO_TOOLTIP
        cmbCompType->setToolTip(QApplication::translate("mainWindow", "Type of 2D component", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        lblComposite->setText(QApplication::translate("mainWindow", "2D &Component Data:", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        txtComposite->setToolTip(QApplication::translate("mainWindow", "Data to be encoded in 2D component\n"
"Remember to place [square brackets] around AI data", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        txtComposite->setHtml(QApplication::translate("mainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Your Data Here!</p></body></html>", 0, QApplication::UnicodeUTF8));
        tabMain->setTabText(tabMain->indexOf(tabGeneral), QApplication::translate("mainWindow", "General", 0, QApplication::UnicodeUTF8));
        label_3->setText(QApplication::translate("mainWindow", "&Height:", 0, QApplication::UnicodeUTF8));
        label_8->setText(QApplication::translate("mainWindow", "B&order Width:", 0, QApplication::UnicodeUTF8));
        lblWhitespace->setText(QApplication::translate("mainWindow", "&Whitespace:", 0, QApplication::UnicodeUTF8));
        lblScale->setText(QApplication::translate("mainWindow", "&Printing Scale:", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        heightb->setToolTip(QApplication::translate("mainWindow", "Overall symbol height", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        heightb->setSuffix(QApplication::translate("mainWindow", " X", 0, QApplication::UnicodeUTF8));
        heightb->setPrefix(QString());
#ifndef QT_NO_TOOLTIP
        bwidth->setToolTip(QApplication::translate("mainWindow", "Width of boundary bars or border", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        bwidth->setSuffix(QApplication::translate("mainWindow", " X", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        spnWhitespace->setToolTip(QApplication::translate("mainWindow", "Width of whitespace", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        spnWhitespace->setSuffix(QApplication::translate("mainWindow", " X", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        spnScale->setToolTip(QApplication::translate("mainWindow", "Image scale when output to file", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        spnScale->setSuffix(QString());
        label_9->setText(QApplication::translate("mainWindow", "Border &Type:", 0, QApplication::UnicodeUTF8));
        btype->clear();
        btype->insertItems(0, QStringList()
         << QApplication::translate("mainWindow", "No border", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("mainWindow", "Bind", 0, QApplication::UnicodeUTF8)
         << QApplication::translate("mainWindow", "Box", 0, QApplication::UnicodeUTF8)
        );
#ifndef QT_NO_TOOLTIP
        btype->setToolTip(QApplication::translate("mainWindow", "Add border or box", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_TOOLTIP
        fgcolor->setToolTip(QApplication::translate("mainWindow", "Change ink colour", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        fgcolor->setText(QApplication::translate("mainWindow", "&Foreground Colour", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        bgcolor->setToolTip(QApplication::translate("mainWindow", "Change paper colour", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        bgcolor->setText(QApplication::translate("mainWindow", "&Backgound Colour", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        btnReset->setToolTip(QApplication::translate("mainWindow", "Reset to black on white", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        btnReset->setText(QApplication::translate("mainWindow", "&Reset Colours", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        chkHRTHide->setToolTip(QApplication::translate("mainWindow", "Show text in image", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        chkHRTHide->setText(QApplication::translate("mainWindow", "Show H&uman Readable Text", 0, QApplication::UnicodeUTF8));
        tabMain->setTabText(tabMain->indexOf(tabAppear), QApplication::translate("mainWindow", "Appearance", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        btnAbout->setToolTip(QApplication::translate("mainWindow", "About Zint", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        btnAbout->setText(QApplication::translate("mainWindow", "&About", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        btnSave->setToolTip(QApplication::translate("mainWindow", "Save image to file", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
        btnSave->setText(QApplication::translate("mainWindow", "&Save", 0, QApplication::UnicodeUTF8));
        btnExit->setText(QApplication::translate("mainWindow", "E&xit", 0, QApplication::UnicodeUTF8));
    } // retranslateUi

};

namespace Ui {
    class mainWindow: public Ui_mainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
